;---------Copyright (C) 1999 Dallas Semiconductor Corporation-----------
;
;   COMBBS
;                      
;   COM port 1-Wire communication Setup/Bit/Byte/Reset with basic
;       DS9097 type adapter.
;
    .MODEL small
    .STACK 100H
    .DATA
;
;
PrtAddr db      32 dup (0)            ;array of port 16 addresses
tmp     db      1  dup (0)            ;temporary storage
msk     db      1  dup (0)            ;mask used in touchbyte 'TByt'
SPA     db      2  dup (0)            ;temporary space for serial port address
X       db      2  dup (0)
Y       db      2  dup (0)
sendbit db      1  dup (0)            ;bit to send
inch    db      1  dup (0)            ;input character
psval   db      1  dup (0)            ;


    .CODE
;---------------------------------------------------------------------------
; TSetup - Make sure there is a COM address in BIOS and put it in the list.
;
;         INPUT   AL - COM number (1-4)
;
;         OUTPUT  AL - 1 setup ok
;                      2 setup ok but port shorted (could indicate no brick)
;                      3 COM port does not exhist in BIOS (or not 1-4)
;
;                 AH - destroyed     
;
;                 PrtAddr[COM#] - set to a port address on success
;                 tmp - contents destroyed
;                 SPA - contents destroyed
;
TSetup  proc      near         
        push      dx
        push      bx          
        push      es
        cmp       al,        1        ;if port < 1
        jb        PRTNEX              ;   goto PRTNEX  (Port not exist)
        cmp       al,        4        ;if port < 4
        jbe       RNGOK               ;   goto RNGOK   (Port range ok)
PRTNEX:
        mov       al,        3        ;Error code 3 in AL
        jmp       DONEST
RNGOK:                                
        mov   byte ptr tmp, al        ;keep port number in tmp
        mov       ah,       0         ;PrtAddr[port] = *(40:00+port-1);
        shl       ax,       1
        mov       bx,       0040h
        mov       es,       bx
        mov       bx,       ax
        mov   ax, word ptr es:[bx-2]
        mov   dl, byte ptr tmp
        mov       dh,       0
        shl       dx,       1
        mov       bx,       dx
        mov   word ptr PrtAddr[bx],ax

        cmp   word ptr PrtAddr[bx],0  ;if (!PrtAddr[port])
        je        PRTNEX              ;   goto PRTNEX  (port not exist)
        
        mov       al, byte ptr tmp      
        call      TReset              ;call TReset to check on shorts

        mov       al,       dl        ;look to see if short test was
        mov       bl,       dh        ;  done
        and       al,       bl        ;
        and       al,       03h       ; 
        cmp       al,       0         ;if not shorted
        je        RTTRUE              ;   goto RTTRUE  (return true)
        mov       al,       2         ;else rtval = 2   
        jmp       DONEST              ;
RTTRUE:
        mov       al,       1         ;rtval = TRUE (1)
DONEST:              
        pop       es
        pop       bx                  ;restore stack 
        pop       dx
        ret
TSetup  endp


;---------------------------------------------------------------------------
; TReset - Do a Touchreset on the COM port.
;
;         INPUT   AL - COM number (1-4)   
;
;                 PrtAddr[COM#] must already be set to a port address from 
;                               TSetup.
;
;
;         OUTPUT  AL - 0 no presence found
;                      1 presence detected
;
;                      2 setup ok but port shorted (could indicate no brick)
;                      3 COM port does not exhist in BIOS (or not 1-4)
;
;                 AH - destroyed     
;                   
;                 DX - Contains test information
;
;                 msk - contents destroyed
;                 tmp - contents destroyed
;                 SPA - contents destroyed
;                 X - contents destroyed
; 
;
TReset  proc      near
        push      bx
        push      cx
        mov       cl,       3         ;set return value     
        mov   byte ptr msk, 1         ;do a delay = TRUE

        mov       dl,       al        ;check to see if have a nonzero
        mov       dh,       0
        shl       dx,       1         ;port address
        mov       bx,       dx
        mov   dx,word ptr PrtAddr[bx] ;initialize port
        mov   word ptr SPA,dx
        cmp       dx,       0
        jnz       PortOk     
        mov       dx,       0         ;no test info
        jmp       RtZero1 
PortOk:
        mov   dx,word ptr SPA         ;set DLAB
        add       dx,       3
        mov       al,       131
        out       dx,       al
        mov   dx,word ptr SPA         ;bit rate is 115200
        mov       al,       1
        out       dx,       al
        mov   dx,word ptr SPA
        inc       dx
        mov       al,       0
        out       dx,       al
        mov   dx,word ptr SPA         ;8 dta, 1 stp, no par
        add       dx,       3
        mov       al,       3
        out       dx,       al        
        mov       al,       1         ;6 dta, 1 stp, no par
        out       dx,       al
        mov   dx,word ptr SPA         ;no interrupts
        inc       dx
        mov       al,       0
        out       dx,       al
        mov   dx,word ptr SPA         ;RTS and DTR on
        add       dx,       4
        mov       al,       3
        out       dx,       al  
        mov   dx,word ptr SPA         ;enable 16550 fifos 
        add       dx,       2
        mov       al,       0C7h
        out       dx,       al  
ClrBuf1:
        mov   dx,word ptr SPA         ;loop to clear the output buffers
        add       dx,       5
        in        al,dx
        mov       ah,       0
        and       ax,       96
        mov   word ptr tmp, ax
        cmp   word ptr tmp, 96
        jne       ClrBuf1
        jmp       ChkWhile
Flush1:
        mov   dx,word ptr SPA         ;flush the input buffer
        in        al,       dx
        mov       ah,       0
        mov   word ptr X,ax
ChkWhile:
        mov   dx,word ptr SPA
        add       dx,       5
        in        al,       dx
        mov       ah,       0
        test      ax,       1
        jne       Flush1

        mov   dx,word ptr SPA         ;set DLAB
        add       dx,       3
        mov       al,       131
        out       dx,       al
        mov   dx,word ptr SPA         ;baud rate is 10473
        inc       dx
        mov       al,       0
        out       dx,       al
        mov   dx,word ptr SPA
        mov       al,       11
        out       dx,       al
        mov   dx,word ptr SPA         ;8 dta, 1 stp, no par
        add       dx,       3
        mov       al,       3
        out       dx,       al
        mov   dx,word ptr SPA         ;enable 16550 fifos 
        add       dx,       2
        mov       al,       0C7h
        out       dx,       al  
        mov   dx,word ptr SPA         ;send the reset pulse
        mov       al,       240
        out       dx,       al

        mov       al,       0         ; get start time
        out       043h,     al        ; freeze value in timer

        in        al,       040h      ; read LSByte of timer
        mov       bl,       al        ;
        in        al,       040h      ; read MSByte                        
        mov       bh,       al        ;         
ChrBkLp1:
        mov   dx,word ptr SPA         ;wait until character back or timeout
        add       dx,       5
        in        al,       dx
        mov       ah,       0
        mov   word ptr Y,ax
        and       ax,       1
        cmp       ax,       0         ;loop test for input character flag
        jne       GetChr              ;or system clock timeout
        jmp       CheckT           
GetChr:
        mov   dx,word ptr SPA
        in        al,       dx
        mov       ah,       0         ;get the character
        mov   word ptr X,ax               
        
        mov   ax,word ptr Y
        and       ax,       018h      ; check for framing error or break
        cmp       ax,       0
        je        NoError
        jmp       CheckSht            ;         
NoError: 
        mov   byte ptr msk, 0         ; dodelay = 0
        mov   ax, word ptr X
        cmp   al, 0F0h                ; check for no parts
        jne       TRReg               ; 
        mov       cl,       0         ; no parts
        jmp       TREnd               ; 
TRReg:                  
        mov       cl,       1         ; 
        jmp       TREnd                       
CheckSht:                             ; 
        mov   ax, word ptr X          ;  look at char     
        cmp   al, 0h                  ;  if char is 0
        jne       TREnd               ;         
        mov   ax,word ptr Y
        and       ax,       060h      ;  check for trans hold, shift empty
        cmp       ax,       0         ;  
        je        TREnd               ;  
        mov       cl,       2         ; 
        jmp       TREnd               ; 
CheckT:                 
        mov       al,       0         ; get current time
        out       043h,     al        ; freeze value in timer
         
        in        al,       040h      ; read LSByte of timer
        mov       dl,       al        ;
        in        al,       040h      ; read MSByte                        
        mov       dh,       al        ; 

        mov       ax,       bx        ; 
        cmp       ax,       dx        ;  
        jg        DoSubt              ; 
        mov       ax,       dx        ;  then flip
        mov       dx,       bx        ; 
        sub       ax,       dx        ;   (now - start)
        mov       dx,       ax        ;  
        mov       ax,       0FFFFh    ;   
DoSubt:                               ;         
        sub       ax,       dx        ; 
        cmp       ax,       02AF8h    ; if result < 11000 then loop 
        jg        NoDelay             ;  no delay if timed out
        jmp       ChrBkLp1            ;  
TREnd:                      
        cmp   byte ptr msk, 0         ;  check if need to do delay
        je    NoDelay                 ; 
        
        mov       al,       0         ; get start time
        out       043h,     al        ; freeze value in timer
        in        al,       040h      ; read LSByte of timer
        mov       bl,       al        ;
        in        al,       040h      ; read MSByte                        
        mov       bh,       al        ;         
PollLoop:        
        mov       dx,word ptr SPA     ; set up address for eating chars
        in        al,       dx        ; try to eat a char
        
        mov       al,       0         ; get current time
        out       043h,     al        ; freeze value in timer
         
        in        al,       040h      ; read LSByte of timer
        mov       dl,       al        ;
        in        al,       040h      ; read MSByte                        
        mov       dh,       al        ; 

        mov       ax,       bx        ; 
        cmp       ax,       dx        ;  
        jg        DoSubt2             ; 
        mov       ax,       dx        ;  then flip
        mov       dx,       bx        ; 
        sub       ax,       dx        ;   (now - start)
        mov       dx,       ax        ;  
        mov       ax,       0FFFFh    ;   
DoSubt2:                              ;         
        sub       ax,       dx        ; 
        cmp       ax,       02AF8h    ; if result < 11000 then loop 
        jl        PollLoop            ;
NoDelay:                              ; 
        mov   dx,word ptr SPA         ;initialize port
        add       dx,       3
        mov       al,       131
        out       dx,       al
        mov   dx,word ptr SPA         ;bit rate is 115200
        mov       al,       1
        out       dx,       al
        mov   dx,word ptr SPA
        inc       dx
        mov       al,       0
        out       dx,       al
        mov   dx,word ptr SPA         ;8 dta, 1 stp, no par
        add       dx,       3
        mov       al,       3
        out       dx,       al
        mov       al,       1         ;6 dta, 1 stp, no par
        out       dx,       al
        mov   dx,word ptr SPA         ;enable 16550 fifos 
        add       dx,       2
        mov       al,       0C7h
        out       dx,       al  

        mov       dx,     00700h      ;set flags to show 1-wire test done

        cmp       cl,       3         ;if shorted then set short flags
        jne       ChkAl2              ;else check for alarm
        mov       dl,       03h
        jmp       DnFlags
ChkAl2:
        cmp       cl,       2         ;if alarming set alarm flag
        jne       DnFlags             ;else done with flags
        mov       dl,       04h
DnFlags:
        cmp       cl,       1         ;if (rtval == 1 || rtval == 2)
        je        RtOne1              ;   return 1;
        cmp       cl,       2
        jne       RtZero1
RtOne1:
        mov       al,       1
        jmp       TRPop
RtZero1:
        mov       al,       0
TRPop:
        pop       cx
        pop       bx
        ret
TReset  endp


;---------------------------------------------------------------------------
; TByt - Do a TouchByte on the COM port.
;
;         INPUT   AL - byte to send
;
;                 AH - COM number (1-4)   
;                 
;                 PrtAddr[COM#] must already be set to a port address from 
;                               TSetup.
;
;
;         OUTPUT  AL - byte returned
;
;                 AH - destroyed     
;                   
;                 msk - contents destroyed
;                 SPA - contents destroyed
;                 inch - contents destroyed
;                 psval - contents destroyed
;                 sendbit - contents destroyed
;
TByt    proc      near
        push      bx     
        push      dx
        mov   byte ptr inch,0         ;set inch=0
        mov   byte ptr msk,1          ;set msk=1
        mov   byte ptr psval,al       ;put output value in psval
        
        mov       dl,       ah        ;check to see if have a nonzero
        mov       dh,       0
        shl       dx,       1         ;port address
        mov       bx,       dx
        mov   dx,word ptr PrtAddr[bx] ;initialize port
        mov   word ptr SPA,dx
        cmp       dx,       0
        jnz       PortOk2     
        mov    byte ptr inch,0        ;no data
        jmp       ClrTst
PortOk2:
        mov       al,       0         ; get start time
        out       043h,     al        ; freeze value in timer

        in        al,       040h      ; read LSByte of timer
        mov       bl,       al        ;
        in        al,       040h      ; read MSByte                        
        mov       bh,       al        ;
ClrBuf3:
        mov   dx,word ptr SPA         ;loop to clear buffers
        add       dx,       5
        in        al,       dx
        mov       ah,       0
        and       ax,       96
        cmp       ax,       96
        jne       ClrBuf3
        jmp       ChkWhil2
Flush2:
        mov   dx,word ptr SPA         ;loop to flush input
        in        al,       dx
ChkWhil2:
        mov   dx,word ptr SPA
        add       dx,       5
        in        al,       dx
        mov       ah,       0
        test      ax,       1
        jne       Flush2

        test  byte ptr psval,1        ;ready first bit to send out
        je        Skip1               ;sendbit = (outch & 0x1) ? Write1 : Write0
        mov   al, 0FFh                ; 
        jmp       LdSndBit            ;goto load send bit
Skip1:
        mov   al, 000h                ;
LdSndBit:
        mov   byte ptr sendbit,al
SndLoop:
        mov   dx,word ptr SPA         ;loop to send 8 bits
        mov   al,byte ptr sendbit
        out       dx,       al        ;send out the bit

        shl   byte ptr msk,1          ;get next bit ready to go out (msk <<= 1)
        mov   dl, byte ptr msk        ;sendbit = (outch & msk) ? Write1 : Write0
        test  byte ptr psval,dl
        je        Skip2
        mov   al, 0FFh                ; 
        jmp       LdSndB2
Skip2:
        mov   al, 000h                ;
LdSndB2:
        mov   byte ptr sendbit,al

        shr   byte ptr inch,1         ;shift input char over ready for next bit
LkLoop:
        mov       al,       0         ; get current time
        out       043h,     al        ; freeze value in timer
         
        in        al,       040h      ; read LSByte of timer
        mov       dl,       al        ;
        in        al,       040h      ; read MSByte                        
        mov       dh,       al        ; 
        
        mov       ax,       bx        ; 
        cmp       ax,       dx        ;  
        jg        DoSubt3             ; 
        mov       ax,       dx        ;  then flip
        mov       dx,       bx        ; 
        sub       ax,       dx        ;   (now - start)
        mov       dx,       ax        ;  
        mov       ax,       0FFFFh    ;   
DoSubt3:                              ;         
        sub       ax,       dx        ; 
        cmp       ax,       01194h    ; if result < 4500 then loop 
        jl        ChkFrBt  
ErrQuit:
        mov   byte ptr inch, 0        ; return with 0x00  cuz shorted
        jmp       ClrTst              ;
ChkFrBt:
        mov   dx,word ptr SPA         ;check for incoming bit
        add       dx,       5         ;if ( inportb(SPA+5) & 0x01 )
        in        al,       dx
        test      al,       1
        je        LkLoop

        mov   dx,word ptr SPA         ;get the bit and place in inch
        in        al,       dx        ;inch |= ((inportb(SPA) & 0x01) ? 0x80 : 0x00)
        
        and       al,       01h       ;
        cmp       al,       0         ;
        je        ChkWhil3            ;its zero so don't do anything to inch
        mov       al,       080h      ;                                                          
LdSndB3:
        or    byte ptr inch,al
ChkWhil3:
        cmp   byte ptr msk,0          ;check if mask = 0 stop loop else
        je        ClrTst              ; 
        jmp       SndLoop             ;  loop
ClrTst:                               ; 
        mov   al,byte ptr inch        ;set return value
        pop       dx
        pop       bx
        ret
TByt   endp


;---------------------------------------------------------------------------
; TBit - Do a TouchBit on the COM port.
;
;         INPUT   AL - bit to send (msbit)
;
;                 AH - COM number (1-4)   
;                 
;                 PrtAddr[COM#] must already be set to a port address from 
;                               TSetup.
;
;
;         OUTPUT  AL - bit returned (msbit)
;
;                 AH - destroyed     
;                   
;                 SPA - contents destroyed
;                 psval - contents destroyed
;                 sendbit - contents destroyed
;
TBit    proc      near
        push      bx   
        push      dx

        mov   byte ptr psval,al       ;put output value in psval
        
        mov       dl,       ah        ;check to see if have a nonzero
        mov       dh,       0
        shl       dx,       1         ;port address
        mov       bx,       dx
        mov   dx,word ptr PrtAddr[bx] ;initialize port
        mov   word ptr SPA,dx
        cmp       dx,       0
        jnz       PortOk3     
        mov       al,       0         ;no data
        jmp       StRtnBit
PortOk3:
        mov       al,       0         ; get start time
        out       043h,     al        ; freeze value in timer

        in        al,       040h      ; read LSByte of timer
        mov       bl,       al        ;
        in        al,       040h      ; read MSByte                        
        mov       bh,       al        ;
ClrBuf4:
        mov   dx,word ptr SPA         ;clear buffers
        add       dx,       5
        in        al,       dx
        mov       ah,       0
        and       ax,       96
        cmp       ax,       96
        jne       ClrBuf4
        jmp       ChkWhil4
Flush4:
        mov   dx,word ptr SPA         ;flush input buffer
        in        al,       dx
ChkWhil4:
        mov   dx,word ptr SPA
        add       dx,       5
        in        al,       dx
        mov       ah,       0
        test      ax,       1
        jne       Flush4

        test  byte ptr psval,1        ;set the output byte (sendbit)
        je        Skip4               ;sendbit = (psval & 0x1) ? Write1 : Write0
        mov   al, 0FFh                ; 
        jmp       LdSndB4
Skip4:
        mov   al, 000h                ;
LdSndB4:
        mov   byte ptr sendbit,al
        mov   dx,word ptr SPA         ;set out the bit
        mov   al,byte ptr sendbit     ;outportb(SPA,sendbit)
        out       dx,       al
LkLoop2:
        mov       al,       0         ; get current time
        out       043h,     al        ; freeze value in timer
         
        in        al,       040h      ; read LSByte of timer
        mov       dl,       al        ;
        in        al,       040h      ; read MSByte                        
        mov       dh,       al        ; 
        
        mov       ax,       bx        ; 
        cmp       ax,       dx        ;  
        jg        DoSubt4             ; 
        mov       ax,       dx        ;  then flip
        mov       dx,       bx        ; 
        sub       ax,       dx        ;   (now - start)
        mov       dx,       ax        ;  
        mov       ax,       0FFFFh    ;   
DoSubt4:                              ;         
        sub       ax,       dx        ; 
        cmp       ax,       01194h    ; if result < 4500 then loop 
        jl        ChkFrBt2   
        mov       al,       0
        jmp       StRtnBit
ChkFrBt2:
        mov   dx,word ptr SPA         ;check for incoming bit
        add       dx,       5         ;if ( inportb(SPA+5) & 0x01 )
        in        al,       dx
        test      al,       1
        je        LkLoop2             ;else loop

        mov   dx,word ptr SPA         ;bit found so read in
        in        al,       dx
        and       al,       01h       ;
StRtnBit:                             ;
        pop       dx
        pop       bx
        ret
TBit    endp


;---------------------------------------------------------------------------
; PrHex - Print the value in AL to standard out as a 2 character
;         hex value.
;
;         INPUT   AL - byte to print
;
;         OUTPUT  two characters to standard out
;
PrHex   proc      near
        push cx
        push ax
        push dx
        
        mov  cl,  al
        shr  al, 4

        cmp  al, 9
        ja   AddExtra1

        mov  ah,  030h
        jmp  DoAdd1

AddExtra1:
        mov  ah,  037h
DoAdd1:
        add  al,  ah
       
        mov  dl,  al
        mov  ah,  2
        int  21h

        mov  al, cl
        and  al, 0Fh
       
        cmp  al, 9
        ja   AddExtra2

        mov  ah,  030h
        jmp  DoAdd2
AddExtra2:
        mov  ah,  037h
DoAdd2:
        add  al,  ah

        mov  dl,  al
        mov  ah,  2
        int  21h

        pop dx
        pop ax
        pop cx
        ret
PrHex   endp


;---------------------------------------------------------------------------
; Test main that sets up COM1 and then does a reset/read rom command/and
; then 8 read bytes to read the ROM of a single device.
;
TESTPROG:
;
;
        mov  ax,@data
        mov  ds,ax                  ;set DS to point to the data segment
                       
        mov  al,   1                ;do Setup on COM1
        call TSetup                           
        call PrHex                  ;print result in al
       
        mov  dl,  02Ch              ; ','
        mov  ah,  2
        int  21h
                  
        mov  al,   1                ;TouchReset on COM1
        call TReset                           
        call PrHex                  ;print result in al

        mov  dl,  02Ch              ; ','
        mov  ah,  2
        int  21h

        mov  al,  033h              ;TouchByte 33 (read ROM command)
        mov  ah,  1                 ;on COM1          
        call TByt                           
        call PrHex                  ;print result in al

        mov  dl,  02Ch              ; ','
        mov  ah,  2
        int  21h

        mov  CX, 8                  ;loop to read 8 bytes

StartLoop:
        mov  al,  0FFh              ;TouchByte FF (read 8 bits)
        mov  ah,  1                 ;on COM1          
        call TByt                           
        call PrHex                  ;print result in al

        loop StartLoop
       
        mov  ah,4ch                 ;DOS terminate program function
        INT     21H                 ;

;
;CODE   ENDS
;
        END     TESTPROG

